@@ -1,20 +1,15 @@
 /*
-** Zabbix
-** Copyright (C) 2001-2022 Zabbix SIA
+** Copyright (C) 2001-2024 Zabbix SIA
 **
-** This program is free software; you can redistribute it and/or modify
-** it under the terms of the GNU General Public License as published by
-** the Free Software Foundation; either version 2 of the License, or
-** (at your option) any later version.
+** This program is free software: you can redistribute it and/or modify it under the terms of
+** the GNU Affero General Public License as published by the Free Software Foundation, version 3.
 **
-** This program is distributed in the hope that it will be useful,
-** but WITHOUT ANY WARRANTY; without even the implied warranty of
-** MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
-** GNU General Public License for more details.
+** This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
+** without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
+** See the GNU Affero General Public License for more details.
 **
-** You should have received a copy of the GNU General Public License
-** along with this program; if not, write to the Free Software
-** Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
+** You should have received a copy of the GNU Affero General Public License along with this program.
+** If not, see <https://www.gnu.org/licenses/>.
 **/
 
 
@@ -24,6 +19,11 @@ const TABFILTER_EVENT_NEWITEM = 'newitem.tabfilter';
 
 class CTabFilter extends CBaseComponent {
 
+	/**
+	 * @type {CSortable|null}
+	 */
+	#tabs_sortable = null;
+
 	constructor(target, options) {
 		super(target);
 		this._options = options;
@@ -36,11 +36,14 @@ class CTabFilter extends CBaseComponent {
 		this._fetch = {};
 		this._idx_namespace = options.idx;
 		this._timeselector = null;
+		this._csrf_token = this._options.csrf_token;
 
 		this.init(options);
 		this.registerEvents();
 		this.initItemUnsavedState(this._active_item, this._active_item._data);
 
+		this.scrollIntoView(this._items[options.selected]);
+
 		if (this._timeselector instanceof CTabFilterItem) {
 			this._timeselector._data = options.timeselector;
 			this.updateTimeselector(this._active_item, this._timeselector._data.disabled);
@@ -56,7 +59,7 @@ class CTabFilter extends CBaseComponent {
 			options.data[options.selected].expanded = true;
 		}
 
-		for (const template of this._target.querySelectorAll('[type="text/x-jquery-tmpl"][data-template]')) {
+		for (const template of this._target.querySelectorAll('[data-template]')) {
 			this._templates[template.getAttribute('data-template')] = template;
 		}
 
@@ -68,7 +71,6 @@ class CTabFilter extends CBaseComponent {
 			}
 
 			if (options.selected == index) {
-				item.renderContentTemplate();
 				this.setSelectedItem(item);
 
 				if (options.expanded) {
@@ -78,23 +80,23 @@ class CTabFilter extends CBaseComponent {
 
 			index++;
 		}
+
+		this.#updateSeparators();
 	}
 
 	/**
 	 * Ensures item label is visible in tab filter navigation.
 	 *
-	 * @param {CTabfilterItem} item    Filter item object.
+	 * @param {CTabFilterItem} item  Filter item object.
 	 */
 	scrollIntoView(item) {
-		let scrollable_parent = item._target.closest('.ui-sortable-container').parentNode;
-
-		scrollable_parent.scrollLeft = item._target.parentNode.offsetLeft - item._target.parentNode.clientWidth;
+		setTimeout(() => this.#tabs_sortable.scrollIntoView(item._target.parentNode, {immediate: true}));
 	}
 
 	/**
 	 * Render filter with profiles stored data to hidden container to get source url for unsaved state comparison.
 	 *
-	 * @param {CTabfilterItem} item    Selected filter object.
+	 * @param {CTabFilterItem} item    Selected filter object.
 	 * @param {object} filter_data     Selected filter object filter data.
 	 */
 	initItemUnsavedState(item, filter_data) {
@@ -121,18 +123,16 @@ class CTabFilter extends CBaseComponent {
 		let index = this._items.indexOf(item);
 
 		if (index != -1) {
+			item.delete();
+			this._items.splice(index, 1);
+			this._items.forEach((item, index) => item._index = index);
+
 			this.setSelectedItem(this._items[index - 1]);
 
 			if (item._expanded) {
 				this._active_item.setExpanded();
 			}
 
-			item.delete();
-			delete this._items[index];
-			this._items.splice(index, 1);
-			this._items.forEach((item, index) => {
-				item._index = index;
-			});
 			this._active_item.setBrowserLocation(this._active_item.getFilterParams());
 		}
 	}
@@ -201,11 +201,15 @@ class CTabFilter extends CBaseComponent {
 			return;
 		}
 
+		this._target.querySelector('a.btn-time').textContent = item.hasCustomTime()
+			? item.getCustomTimeLabel()
+			: this._timeselector._data.label;
+
 		let disabled = disable || (!this._options.support_custom_time || item.hasCustomTime()),
 			buttons = {
-				decrement_button: this._target.querySelector('button.btn-time-left'),
-				increment_button: this._target.querySelector('button.btn-time-right'),
-				zoomout_button: this._target.querySelector('button.btn-time-out')
+				decrement_button: this._target.querySelector('button.js-btn-time-left'),
+				increment_button: this._target.querySelector('button.js-btn-time-right'),
+				zoomout_button: this._target.querySelector('button.btn-time-zoomout')
 			};
 
 		this._timeselector.setDisabled(disabled);
@@ -253,6 +257,7 @@ class CTabFilter extends CBaseComponent {
 		}
 
 		body.idx = this._idx_namespace + '.' + property;
+		body[CSRF_TOKEN_NAME] = this._csrf_token;
 
 		if (property !== 'properties') {
 			this._fetch[property] = new AbortController();
@@ -303,7 +308,6 @@ class CTabFilter extends CBaseComponent {
 
 		if (item !== this._timeselector) {
 			item._target.setAttribute('tabindex', 0);
-			this.scrollIntoView(item);
 			item.setBrowserLocationToApplyUrl();
 		}
 
@@ -316,6 +320,41 @@ class CTabFilter extends CBaseComponent {
 				}
 			}
 		}
+
+		this.#updateSeparators();
+	}
+
+	/**
+	 * Update separated state of items (whether to visually separate one item from another).
+	 */
+	#updateSeparators() {
+		for (let index = 0; index < this._items.length; index++) {
+			this._items[index].setSeparated(
+				index > 0 && !this._items[index].isSelected() && !this._items[index - 1].isSelected()
+			);
+		}
+	}
+
+	/**
+	 * Update separated state of items (whether to visually separate one item from another) for dragging.
+	 *
+	 * @param {number} index     Index of item being dragged.
+	 * @param {number} index_to  Overtake index.
+	 */
+	#updateSeparatorsForDragging(index, index_to) {
+		const items = [...this._items];
+
+		items.splice(index_to, 0, ...items.splice(index, 1));
+
+		for (let index = 0; index < items.length; index++) {
+			const separated_state = index > 0
+				&& index !== index_to
+				&& index !== index_to + 1
+				&& !items[index].isSelected()
+				&& !items[index - 1].isSelected();
+
+			this.#tabs_sortable.mutate(() => items[index].setSeparated(separated_state));
+		}
 	}
 
 	/**
@@ -384,6 +423,8 @@ class CTabFilter extends CBaseComponent {
 						|| (item !== this._timeselector && this._active_item === this._timeselector)
 					);
 
+				item.setFocused();
+
 				if (item !== this._timeselector) {
 					if (item.isSelected()) {
 						this.profileUpdate('expanded', {
@@ -393,7 +434,6 @@ class CTabFilter extends CBaseComponent {
 						});
 					}
 					else {
-						item.setFocused();
 						item.initUnsavedState();
 						this.profileUpdate('selected', {
 							value_int: item._index
@@ -401,8 +441,6 @@ class CTabFilter extends CBaseComponent {
 							this._options.selected = item._index;
 						});
 					}
-
-					this.scrollIntoView(item);
 				}
 
 				if (item !== this._active_item) {
@@ -410,7 +448,6 @@ class CTabFilter extends CBaseComponent {
 					this.collapseAllItemsExcept(item);
 
 					if (expand) {
-						item.setFocused();
 						item.fire(TABFILTERITEM_EVENT_EXPAND);
 					}
 				}
@@ -464,34 +501,66 @@ class CTabFilter extends CBaseComponent {
 			},
 
 			/**
-			 * UI sortable update event handler. Updates tab sorting in profile.
+			 * Listener for tabs event {CSortable.EVENT_DRAG_START}.
+			 *
+			 * Updates separators of items.
+			 *
+			 * @param {Object} ev
 			 */
-			tabSortChanged: (ev, ui) => {
-				// Update order of this._items array.
-				let from,
-					to,
-					item_moved;
-				const target = ui.item[0].querySelector('[data-target] .tabfilter-item-link');
+			tabsDragStart: (ev) => {
+				this._items[ev.detail.index].setFocused();
 
-				this._items.forEach((item, index) => from = (item._target === target) ? index : from);
-				this._target.querySelectorAll('nav [data-target] .tabfilter-item-link')
-					.forEach((elm, index) => to = (elm === target) ? index : to);
+				this.#updateSeparatorsForDragging(ev.detail.index, ev.detail.index);
+			},
 
-				item_moved = this._items[from];
-				this._items.splice(from, 1);
-				this._items.splice(to, 0, item_moved);
+			/**
+			 * Listener for tabs event {CSortable.EVENT_DRAG_OVERTAKE}.
+			 *
+			 * Updates separators of items.
+			 *
+			 * @param ev
+			 */
+			tabsDragOvertake: (ev) => {
+				this.#updateSeparatorsForDragging(ev.detail.index, ev.detail.index_to);
+			},
 
-				// Tab order changed, update changes via ajax.
-				let value_str = this._items.map((item) => item._index).join(',');
+			/**
+			 * Listener for tabs event {CSortable.EVENT_DRAG_END}.
+			 *
+			 * Re-focuses the item marked as focused and updates separators of items.
+			 *
+			 * @param ev
+			 */
+			tabsDragEnd: (ev) => {
+				this._items[ev.detail.index].setFocused();
+
+				if (ev.detail.index_to == ev.detail.index) {
+					this.#updateSeparators();
+				}
+			},
+
+			/**
+			 * Listener for tabs event {CSortable.EVENT_SORT}.
+			 *
+			 * Updates tab sorting and updates separators of items.
+			 *
+			 * @param ev
+			 */
+			tabsSort: (ev) => {
+				this._items.splice(ev.detail.index_to, 0, ...this._items.splice(ev.detail.index, 1));
+
+				this.#updateSeparators();
+
+				this.#tabs_sortable.enableSorting(false);
 
 				this.profileUpdate('taborder', {
-					value_str: value_str
+					value_str: this._items.map(item => item._index).join(',')
 				})
-				.then(() => {
-					this._items.forEach((item, index) => {
-						item._index = index;
+					.then(() => {
+						this._items.forEach((item, index) => item._index = index);
+
+						this.#tabs_sortable.enableSorting();
 					});
-				});
 			},
 
 			/**
@@ -581,7 +650,7 @@ class CTabFilter extends CBaseComponent {
 								if (this._active_item !== this._items[0]) {
 									this._items[0].fire(TABFILTERITEM_EVENT_SELECT);
 									// Set selected item focus after popup menu will focus it opener element (used for ESC).
-									setTimeout(() => this._items[0].setFocused(), 0);
+									setTimeout(() => this._items[0].setFocused());
 								}
 							}
 						}]
@@ -598,7 +667,7 @@ class CTabFilter extends CBaseComponent {
 								if (this._active_item !== item) {
 									item.fire(TABFILTERITEM_EVENT_SELECT);
 									// Set selected item focus after popup menu will focus it opener element (used for ESC).
-									setTimeout(() => item.setFocused(), 0);
+									setTimeout(() => item.setFocused());
 								}
 							}
 						});
@@ -620,14 +689,14 @@ class CTabFilter extends CBaseComponent {
 			 * Action on 'Update' button press.
 			 */
 			buttonUpdateAction: () => {
-				var params = this._active_item.getFilterParams();
+				var params = this._active_item.getFilterParams(false);
 
 				this.profileUpdate('properties', {
 					idx2: this._active_item._index,
 					value_str: params.toString()
 				})
 				.then(() => {
-					this._active_item.updateApplyUrl();
+					this._active_item.updateApplyUrl(false);
 					this._active_item.setBrowserLocation(params);
 					this._active_item.resetUnsavedState();
 				});
@@ -650,8 +719,8 @@ class CTabFilter extends CBaseComponent {
 			buttonApplyAction: () => {
 				this._active_item.unsetExpandedSubfilters();
 				this._active_item.emptySubfilter();
-				this._active_item.updateUnsavedState();
-				this._active_item.updateApplyUrl();
+				this._active_item.updateUnsavedState(false);
+				this._active_item.updateApplyUrl(false);
 				this._active_item.setBrowserLocationToApplyUrl();
 			},
 
@@ -660,7 +729,7 @@ class CTabFilter extends CBaseComponent {
 			 */
 			buttonResetAction: () => {
 				let current_url = new Curl(),
-					url = new Curl('zabbix.php', false);
+					url = new Curl('zabbix.php');
 
 				url.setArgument('action', current_url.getArgument('action'));
 				url.setArgument('filter_reset', 1);
@@ -704,16 +773,6 @@ class CTabFilter extends CBaseComponent {
 
 					cancelEvent(ev);
 				}
-			},
-
-			/**
-			 * Scroll horizontally with mouse wheel handler for sortable items container.
-			 */
-			mouseWheelHandler: (container, ev) => {
-				if ((ev.deltaY < 0 && container.scrollLeft > 0)
-						|| (ev.deltaY > 0 && container.scrollLeft < container.scrollWidth - container.clientWidth)) {
-					container.scrollBy({left: ev.deltaY});
-				}
 			}
 		}
 
@@ -725,37 +784,14 @@ class CTabFilter extends CBaseComponent {
 				.on(TABFILTERITEM_EVENT_URLSET, () => this.fire(TABFILTER_EVENT_URLSET));
 		}
 
-		$('.ui-sortable-container', this._target).sortable({
-			items: '.tabfilter-item-label:not(:first-child)',
-			update: this._events.tabSortChanged,
-			stop: (_, ui) => {
-				const $item = ui.item;
-
-				/**
-				 * Remove inline style position, left and top that stay after sortable.
-				 * This styles broken tabs layout.
-				 */
-				if ($item.css('position') === 'relative') {
-					$item.css({
-						'position': '',
-						'left': '',
-						'top': ''
-					});
-				}
-			},
-			axis: 'x',
-			containment: 'parent'
-		});
-
-		const container = this._target.querySelector('.ui-sortable-container').parentNode;
-
-		try {
-			addEventListener('test', null, {get passive() {}});
-			container.addEventListener('wheel', ev => this._events.mouseWheelHandler(container, ev), {passive: true});
-		}
-		catch(e) {
-			container.addEventListener('wheel', ev => this._events.mouseWheelHandler(container, ev));
-		}
+		this.#tabs_sortable = new CSortable(this._target.querySelector('.tabfilter-tabs'), {
+			is_horizontal: true,
+			freeze_start: 1
+		})
+			.on(CSortable.EVENT_DRAG_START, this._events.tabsDragStart)
+			.on(CSortable.EVENT_DRAG_OVERTAKE, this._events.tabsDragOvertake)
+			.on(CSortable.EVENT_DRAG_END, this._events.tabsDragEnd)
+			.on(CSortable.EVENT_SORT, this._events.tabsSort);
 
 		for (const action of this._target.querySelectorAll('nav [data-action]')) {
 			action.addEventListener('click', this._events[action.getAttribute('data-action')]);
